"""
:authors: Runner
:license: Apache License, Version 2.0, see LICENSE file

:copyright: (c) 2021 Runner
"""

from .TZ1 import *
from .TZ2 import *
from .TZ3 import *
from .TZ4 import *
from .TZ5 import *
from .TZ6 import *
from .TZ7 import *
from .TZ8 import *

__author__ = 'Runner'
__version__ = '1.0.1'
__email__ = 'levintsov.v.v@gmail.com'